package com.adlerneves.git.teste_admissional_2020_12_14.activities

import android.os.Bundle
import android.view.View
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import com.adlerneves.git.teste_admissional_2020_12_14.R
import com.adlerneves.git.teste_admissional_2020_12_14.adapters.RedditPostRecyclerViewAdapter
import com.adlerneves.git.teste_admissional_2020_12_14.databinding.ActivitySubredditBinding
import com.adlerneves.git.teste_admissional_2020_12_14.endpoints.SubredditEndpoint

class SubredditActivity : AppCompatActivity() {
    private lateinit var binding: ActivitySubredditBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivitySubredditBinding.inflate(layoutInflater)
        val view = binding.root
        setContentView(view)
        val subredditName: String? = intent.getStringExtra("subreddit")
        if (subredditName == null || subredditName.isEmpty()) {
            finish()
            return
        }
        supportActionBar?.title = "r/$subredditName"
        binding.loadingText.visibility = View.VISIBLE
        binding.recyclerView.visibility = View.GONE
        binding.recyclerView.layoutManager = LinearLayoutManager(this)
        binding.recyclerView.adapter = RedditPostRecyclerViewAdapter(listOf())
        SubredditEndpoint(subredditName).getAndExecuteWhenReady({
            if (!this.isFinishing || !this.isDestroyed) {
                this.runOnUiThread {
                    binding.loadingText.visibility = View.VISIBLE
                    binding.recyclerView.visibility = View.GONE
                    AlertDialog
                        .Builder(this)
                        .setTitle(getString(R.string.error_while_loading))
                        .setMessage(it.toString())
                        .setPositiveButton("Ok") { dialog, _ -> dialog.dismiss() }
                        .setOnDismissListener { this.finish() }
                        .create()
                        .show()
                }
            }
        }) {
            this.runOnUiThread {
                binding.loadingText.visibility = View.GONE
                binding.recyclerView.visibility = View.VISIBLE
                binding.recyclerView.adapter = RedditPostRecyclerViewAdapter(it)
            }
        }
    }
}