package com.adlerneves.git.teste_admissional_2020_12_14.adapters

import android.annotation.SuppressLint
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.text.method.LinkMovementMethod
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.cardview.widget.CardView
import androidx.core.content.ContextCompat.startActivity
import androidx.core.text.HtmlCompat
import com.adlerneves.git.teste_admissional_2020_12_14.R
import com.adlerneves.git.teste_admissional_2020_12_14.helper_extensions.inflate
import com.adlerneves.git.teste_admissional_2020_12_14.helper_extensions.prettifyTimeSpan
import com.adlerneves.git.teste_admissional_2020_12_14.helper_extensions.setResourceDrawable
import com.adlerneves.git.teste_admissional_2020_12_14.helper_extensions.toBitmap
import com.adlerneves.git.teste_admissional_2020_12_14.models.RedditPost


class RedditPostRecyclerViewAdapter(dataListParam: List<RedditPost>) :
    AbstractRecyclerViewAdapterWithData<RedditPost>(
        dataListParam
    ) {
    override fun onCreateViewHolder(
        parent: ViewGroup,
        viewType: Int
    ): DataAndViewHolder<RedditPost> {
        return DataAndViewHolder(parent.inflate(R.layout.list_item_reddit_post))
    }

    override fun onBindViewHolder(
        holder: DataAndViewHolder<RedditPost>,
        position: Int,
        data: RedditPost
    ) {
        val view = holder.itemView
        val card = view.findViewById<CardView>(R.id.card)
        val flairs = view.findViewById<TextView>(R.id.flairs)
        val title = view.findViewById<TextView>(R.id.title)
        val thumbnailImage = view.findViewById<ImageView>(R.id.thumbnailImage)
        val voteCount = view.findViewById<TextView>(R.id.voteCount)
        val messageCount = view.findViewById<TextView>(R.id.messageCount)
        val timePosted = view.findViewById<TextView>(R.id.timePosted)
        val domain = view.findViewById<TextView>(R.id.domain)
        val subreddit = view.findViewById<TextView>(R.id.subreddit)
        val comment = view.findViewById<TextView>(R.id.comment)
        thumbnailImage?.setResourceDrawable(R.drawable.ic_baseline_image_search_24)
        if (data.imageBytes != null) {
            thumbnailImage.visibility = View.VISIBLE
            data.imageBytes?.let { thumbnailImage.setImageBitmap(it.toBitmap()) }
        } else {
            thumbnailImage.visibility = View.GONE
            data.downloadImageBytes({
                it.printStackTrace()
            }) {
                if (data.id == holder.data?.id) {
                    thumbnailImage.visibility = View.VISIBLE
                    thumbnailImage.setImageBitmap(it.toBitmap())
                }
            }
        }
        @SuppressLint("SetTextI18n")
        subreddit.text = "r/${data.subreddit}"
        flairs.text = data.flairs.joinToString(", ")
        if (flairs.text.isEmpty())
            flairs.visibility = View.GONE
        else
            flairs.visibility = View.VISIBLE
        title.text = data.title
        if (!data.contentHtml.isNullOrEmpty()) {
            comment.visibility = View.VISIBLE
            comment.linksClickable = true
            comment.text = HtmlCompat.fromHtml(data.contentHtml, HtmlCompat.FROM_HTML_MODE_LEGACY)
            comment.movementMethod = LinkMovementMethod.getInstance()
        } else
            comment.visibility = View.GONE
        domain.text = data.domain
        voteCount.text = data.balance.toString()
        messageCount.text = data.comments.toString()
        if (data.clickableUrl.isNullOrEmpty())
            card.setOnClickListener { }
        else {
            card.setOnClickListener {
                val intent = Intent(Intent.ACTION_VIEW, Uri.parse(data.clickableUrl))
                startActivity(view.context, intent, Bundle())
            }
        }
        timePosted.text =
            ((System.currentTimeMillis() / 1000L) - data.utcCreated.toLong()).prettifyTimeSpan()
    }
}