package com.adlerneves.git.teste_admissional_2020_12_14.endpoints

import com.adlerneves.git.teste_admissional_2020_12_14.helper_extensions.doAsync
import java.net.HttpURLConnection
import java.net.URL

abstract class AbstractHttpEndpoint<T>(
    val urlString: String,
    val headers: List<Pair<String, String>>,
    val body: ByteArray
) {
    constructor(urlString: String, headers: List<Pair<String, String>>) : this(
        urlString,
        headers,
        byteArrayOf()
    )

    constructor(urlString: String) : this(urlString, listOf<Pair<String, String>>())

    abstract fun processGetResponse(dataBytes: ByteArray): T
    fun getAndExecuteWhenReady(errorCallback: (Throwable) -> Unit, callback: (T) -> Unit) {
        doAsync {
            lateinit var dataBytes: ByteArray
            try {
                val url = URL(urlString)
                val connection = url.openConnection() as HttpURLConnection
                connection.useCaches = false
                connection.connectTimeout = 3000
                connection.readTimeout = 5000
                connection.allowUserInteraction = false
                connection.doOutput = false
                connection.requestMethod = "GET"
                headers.forEach { connection.setRequestProperty(it.first, it.second) }
                connection.connect()
                val responseCode = connection.responseCode
                when (responseCode) {
                    HttpURLConnection.HTTP_OK -> {
                        dataBytes = connection.inputStream.buffered().readBytes()
                    }
                    else -> throw Exception("HTTP code: $responseCode")
                }
                callback(processGetResponse(dataBytes))
            } catch (t: Throwable) {
                t.printStackTrace()
                errorCallback(t)
                return@doAsync
            }
        }
    }
}