package com.adlerneves.git.teste_admissional_2020_12_14.models

import com.adlerneves.git.teste_admissional_2020_12_14.endpoints.SimpleDownloader
import com.adlerneves.git.teste_admissional_2020_12_14.helper_extensions.doAsync
import com.adlerneves.git.teste_admissional_2020_12_14.helper_extensions.optNullableInt
import com.adlerneves.git.teste_admissional_2020_12_14.helper_extensions.optNullableString
import org.json.JSONObject

class RedditPost(
    val id: String,
    val user: String,
    val subreddit: String,
    val title: String,
    val flairs: List<String>,
    val domain: String,
    val balance: Int,
    val comments: Int,
    val utcCreated: Int,
    val content: String?,
    val contentHtml: String?,
    val commentsRelativeUrl: String?,
    val clickableUrl: String?,
    val imagePreviewUrl: String?,
    var imageBytes: ByteArray? = null
) {
    companion object Deserializers {
        fun fromJSONObject(jsonObject: JSONObject): RedditPost {
            return RedditPost(
                jsonObject.getString("name"),
                jsonObject.getString("author"),
                jsonObject.getString("subreddit"),
                jsonObject.getString("title"),
                listOf(
                    when (jsonObject.getBoolean("over_18")) {
                        true -> "nsfw"; false -> ""
                    },
                    jsonObject.optNullableString("link_flair_text") ?: ""
                )
                    .distinct().filterNot { it.isEmpty() },
                jsonObject.optNullableString("domain", "self")!!,
                jsonObject.optNullableInt("ups", 0)!! - jsonObject.optNullableInt("downs", 0)!!,
                jsonObject.optNullableInt("num_comments", 0)!!,
                jsonObject.optNullableInt("created_utc", 0)!!,
                jsonObject.optNullableString("selftext"),
                jsonObject.optNullableString("selftext_html"),
                jsonObject.optNullableString("permalink"),
                jsonObject.optNullableString("url"),
                jsonObject.optNullableString("thumbnail"),
            )
        }
    }

    fun downloadImageBytes(errorCallback: (Throwable) -> Unit, callback: (ByteArray) -> Unit) {
        if (imagePreviewUrl?.startsWith("http") == true && imageBytes == null)
            doAsync {
                SimpleDownloader(imagePreviewUrl).getAndExecuteWhenReady(errorCallback) {
                    this.imageBytes = it
                    callback(it)
                }
            }
    }
}